@@ -132,6 +132,7 @@ from unyt._pint_conversions import convert_pint_units
 from unyt.unit_object import (
     _check_em_conversion,
     _em_conversion,
+    _sanitize_unit_system,
     Unit,
 )
 from unyt.unit_registry import UnitRegistry
@@ -220,7 +221,7 @@ def _bitop_units(unit1, unit2):
         "Bit-twiddling operators are not defined for unyt_array instances")
 
 
-def _coerce_iterable_units(input_object):
+def _coerce_iterable_units(input_object, registry=None):
     if isinstance(input_object, np.ndarray):
         return input_object
     if _iterable(input_object):
@@ -230,7 +231,7 @@ def _coerce_iterable_units(input_object):
                     for _ in input_object]):
                 raise IterableUnitCoercionError(input_object)
             # This will create a copy of the data in the iterable.
-            return unyt_array(input_object)
+            return unyt_array(np.array(input_object), ff, registry=registry)
     return np.asarray(input_object)
 
 
@@ -521,8 +522,7 @@ class unyt_array(np.ndarray):
             pass
         elif _iterable(input_array) and input_array:
             if isinstance(input_array[0], unyt_array):
-                return unyt_array(np.array(input_array, dtype=dtype),
-                                  input_array[0].units, registry=registry)
+                return _coerce_iterable_units(input_array, registry)
 
         # Input array is an already formed ndarray instance
         # We first cast to be our class type
@@ -600,7 +600,8 @@ class unyt_array(np.ndarray):
         units = _sanitize_units_convert(units, self.units.registry)
         if equivalence is None:
             try:
-                conv_data = _check_em_conversion(self.units.expr, units.expr)
+                conv_data = _check_em_conversion(
+                    self.units, units, registry=self.units.registry)
             except MKSCGSConversionError:
                 raise UnitConversionError(self.units, self.units.dimensions,
                                           units, units.dimensions)
@@ -762,7 +763,8 @@ class unyt_array(np.ndarray):
         units = _sanitize_units_convert(units, self.units.registry)
         if equivalence is None:
             try:
-                conv_data = _check_em_conversion(self.units.expr, units.expr)
+                conv_data = _check_em_conversion(
+                    self.units, units, registry=self.units.registry)
             except MKSCGSConversionError:
                 raise UnitConversionError(self.units, self.units.dimensions,
                                           units, units.dimensions)
@@ -890,13 +892,15 @@ class unyt_array(np.ndarray):
         >>> print(E.in_base("mks"))
         2.5e-07 W
         """
+        us = _sanitize_unit_system(unit_system, self)
         try:
-            conv_data = _check_em_conversion(self.units.expr)
+            conv_data = _check_em_conversion(
+                self.units, unit_system=us, registry=self.units.registry)
         except MKSCGSConversionError:
             raise UnitsNotReducible(self.units, unit_system)
         if any(conv_data):
             to_units, (conv, offset) = _em_conversion(
-                self.units, conv_data, unit_system=unit_system)
+                self.units, conv_data, unit_system=us)
         else:
             to_units = self.units.get_base_equivalent(unit_system)
             conv, offset = self.units.get_conversion_factor(to_units)
@@ -1017,7 +1021,7 @@ class unyt_array(np.ndarray):
         -------
         >>> from unyt import km
         >>> (1.0*km).list_equivalencies()
-        spectral: length <-> frequency <-> energy
+        spectral: length <-> spatial_frequency <-> frequency <-> energy
         schwarzschild: mass <-> length
         compton: mass <-> length
         """
@@ -1577,16 +1581,17 @@ class unyt_array(np.ndarray):
                 u1 = Unit(registry=getattr(u0, 'registry', None))
             elif ufunc is power:
                 u1 = inp1
-                if isinstance(u1, np.ndarray):
-                    if isinstance(u1, unyt_array):
-                        if u1.units.is_dimensionless:
-                            pass
-                        else:
-                            raise UnitOperationError(ufunc, u0, u1)
-                    try:
-                        u1 = float(u1)
-                    except TypeError:
+                if inp0.shape != () and inp1.shape != ():
+                    raise UnitOperationError(ufunc, u0, u1)
+                if isinstance(u1, unyt_array):
+                    if u1.units.is_dimensionless:
+                        pass
+                    else:
                         raise UnitOperationError(ufunc, u0, u1)
+                if u1.shape == ():
+                    u1 = float(u1)
+                else:
+                    u1 = 1.0
             unit_operator = self._ufunc_registry[ufunc]
             if unit_operator in (_preserve_units, _comparison_unit,
                                  _arctan2_unit):
@@ -1843,6 +1848,9 @@ class unyt_quantity(unyt_array):
             raise RuntimeError("unyt_quantity instances must be scalars")
         return ret
 
+    def __round__(self):
+        return type(self)(round(float(self)), self.units)
+
 
 def _validate_numpy_wrapper_units(v, arrs):
     if not any(isinstance(a, unyt_array) for a in arrs):
