@@ -53,24 +53,22 @@ from unyt.dimensions import (
     base_dimensions,
     dimensionless,
     temperature,
+    current_mks,
 )
+import unyt.dimensions as dims
 from unyt.equivalencies import equivalence_registry
 from unyt.exceptions import (
     InvalidUnitOperation,
     MKSCGSConversionError,
     UnitConversionError,
-    UnitsNotReducible
+    UnitsNotReducible,
 )
 from unyt._physical_ratios import speed_of_light_cm_per_s
-from unyt._unit_lookup_table import (
-    unit_prefixes,
-    prefixable_units,
-    latex_prefixes,
-)
 from unyt.unit_registry import (
     default_unit_registry,
+    _lookup_unit_symbol,
     UnitRegistry,
-    UnitParseError
+    UnitParseError,
 )
 
 sympy_one = sympify(1)
@@ -84,6 +82,15 @@ global_dict = {
 }
 
 
+def _sanitize_unit_system(unit_system, obj):
+    from unyt.unit_systems import unit_system_registry
+    if hasattr(unit_system, "unit_registry"):
+        unit_system = unit_system.unit_registry.unit_system_id
+    elif unit_system == "code":
+        unit_system = obj.registry.unit_system_id
+    return unit_system_registry[str(unit_system)]
+
+
 def _auto_positive_symbol(tokens, local_dict, global_dict):
     """
     Inserts calls to ``Symbol`` for undefined variables.
@@ -241,7 +248,7 @@ class Unit(object):
                 unit_expr = parse_expr(unit_expr, global_dict=global_dict,
                                        transformations=unit_text_transform)
             except SyntaxError as e:
-                msg = ("Unit expression %s raised an error "
+                msg = ("Unit expression '%s' raised an error "
                        "during parsing:\n%s" % (unit_expr, repr(e)))
                 raise UnitParseError(msg)
         # Simplest case. If user passes a Unit object, just use the expr.
@@ -262,7 +269,7 @@ class Unit(object):
         # Make sure we have an Expr at this point.
         if not isinstance(unit_expr, Expr):
             raise UnitParseError("Unit representation must be a string or "
-                                 "sympy Expr. %s has type %s."
+                                 "sympy Expr. '%s' has type '%s'."
                                  % (unit_expr, type(unit_expr)))
 
         # this is slightly faster if unit_expr is the same object as
@@ -292,7 +299,7 @@ class Unit(object):
                 base_value = float(base_value)
             except ValueError:
                 raise UnitParseError("Could not use base_value as a float. "
-                                     "base_value is '%s' (type %s)."
+                                     "base_value is '%s' (type '%s')."
                                      % (base_value, type(base_value)))
 
             # check that dimensions is valid
@@ -593,7 +600,7 @@ class Unit(object):
         --------
         >>> from unyt import km
         >>> km.units.list_equivalencies()
-        spectral: length <-> frequency <-> energy
+        spectral: length <-> spatial_frequency <-> frequency <-> energy
         schwarzschild: mass <-> length
         compton: mass <-> length
         """
@@ -630,14 +637,10 @@ class Unit(object):
         >>> (g/cm**3).get_base_equivalent('solar')
         Mearth/AU**3
         """
-        from unyt.unit_systems import unit_system_registry
-        if hasattr(unit_system, "unit_registry"):
-            unit_system = unit_system.unit_registry.unit_system_id
-        elif unit_system == "code":
-            unit_system = self.registry.unit_system_id
-        unit_system = unit_system_registry[str(unit_system)]
+        unit_system = _sanitize_unit_system(unit_system, self)
         try:
-            conv_data = _check_em_conversion(self.units.expr)
+            conv_data = _check_em_conversion(
+                self.units, registry=self.registry, unit_system=unit_system)
         except MKSCGSConversionError:
             raise UnitsNotReducible(self.units, unit_system)
         if any(conv_data):
@@ -711,22 +714,26 @@ class Unit(object):
 #
 
 
+# map from dimensions in one unit system to dimensions in other system,
+# canonical unit to convert to in that system, and floating point
+# conversion factor
 em_conversions = {
-    "C": ("esu", 0.1*speed_of_light_cm_per_s, "cgs"),
-    "T": ("gauss", 1.0e4, "cgs"),
-    "Wb": ("Mx", 1.0e8, "cgs"),
-    "A": ("statA", 0.1*speed_of_light_cm_per_s, "cgs"),
-    "V": ("statV", 1.0e-8*speed_of_light_cm_per_s, "cgs"),
-    "ohm": ("statohm", 1.0e9/(speed_of_light_cm_per_s**2), "cgs"),
-    "esu": ("C", 10.0/speed_of_light_cm_per_s, "mks"),
-    "Fr": ("C", 10.0/speed_of_light_cm_per_s, "mks"),
-    "statC": ("C", 10.0/speed_of_light_cm_per_s, "mks"),
-    "gauss": ("T", 1.0e-4, "mks"),
-    "G": ("T", 1.0e-4, "mks"),
-    "Mx": ("Wb", 1.0e-8, "mks"),
-    "statA": ("A", 10.0/speed_of_light_cm_per_s, "mks"),
-    "statV": ("V", 1.0e8/speed_of_light_cm_per_s, "mks"),
-    "statohm": ("ohm", speed_of_light_cm_per_s**2*1.0e-9, "mks"),
+    dims.charge_mks: (dims.charge_cgs, "esu", 0.1*speed_of_light_cm_per_s),
+    dims.charge_cgs: (dims.charge_mks, "C", 10.0/speed_of_light_cm_per_s),
+    dims.magnetic_field_mks: (dims.magnetic_field_cgs, "gauss", 1.0e4),
+    dims.magnetic_field_cgs: (dims.magnetic_field_mks, "T", 1.0e-4),
+    dims.current_mks: (
+        dims.current_cgs, "statA", 0.1*speed_of_light_cm_per_s),
+    dims.current_cgs: (dims.current_mks, "A", 10.0/speed_of_light_cm_per_s),
+    dims.electric_potential_mks: (
+        dims.electric_potential_cgs, "statV",
+        1.0e-8*speed_of_light_cm_per_s),
+    dims.electric_potential_cgs: (
+        dims.electric_potential_mks, "V", 1.0e8/speed_of_light_cm_per_s),
+    dims.resistance_mks: (
+        dims.resistance_cgs, "statohm", 1.0e9/(speed_of_light_cm_per_s**2)),
+    dims.resistance_cgs: (
+        dims.resistance_mks, "ohm", 1.0e-9*speed_of_light_cm_per_s**2)
 }
 
 
@@ -739,9 +746,11 @@ def _em_conversion(orig_units, conv_data, to_units=None, unit_system=None):
     with the new one in the unit expression and multiplying by the scale
     factor.
     """
-    conv_unit, scale = conv_data
+    conv_unit, canonical_unit, scale = conv_data
+    if conv_unit is None:
+        conv_unit = canonical_unit
     new_expr = orig_units.copy().expr.replace(
-        orig_units.expr, scale*conv_unit.expr)
+        orig_units.expr, scale*canonical_unit.expr)
     if unit_system is not None:
         # we don't know the to_units, so we get it directly from the
         # conv_data
@@ -753,24 +762,9 @@ def _em_conversion(orig_units, conv_data, to_units=None, unit_system=None):
     return to_units, conv
 
 
-def _get_em_base_unit(units):
-    unit_str = str(units)
-    if len(unit_str) == 1:
-        return unit_str
-    possible_prefix = unit_str[0]
-    prefix_len = 1
-    if unit_str[:2] == 'da':
-        possible_prefix = 'da'
-        prefix_len += 1
-    if possible_prefix in unit_prefixes:
-        base_unit = unit_str[prefix_len:]
-    else:
-        base_unit = unit_str
-    return base_unit
-
-
 @lru_cache(maxsize=128, typed=False)
-def _check_em_conversion(unit_expr, to_unit_expr=None):
+def _check_em_conversion(unit, to_unit=None, unit_system=None,
+                         registry=None):
     """Check to see if the units contain E&M units
 
     This function supports unyt's ability to convert data to and from E&M
@@ -784,24 +778,37 @@ def _check_em_conversion(unit_expr, to_unit_expr=None):
     trying to convert between CGS & MKS E&M units, it raises an error.
     """
     em_map = ()
-    base_unit = _get_em_base_unit(str(unit_expr))
-    base_to_unit = _get_em_base_unit(str(to_unit_expr))
-    if base_unit == base_to_unit:
+    if unit == to_unit:
         return em_map
-    if base_unit in em_conversions:
-        em_info = em_conversions.get(base_unit, (None,)*2)
-        to_unit = Unit(em_info[0])
-        if ((to_unit_expr is None or
-             Unit(to_unit_expr).dimensions == to_unit.dimensions)):
-            em_map = (to_unit, em_info[1])
+    if unit.dimensions in em_conversions:
+        em_info = em_conversions[unit.dimensions]
+        em_unit = Unit(em_info[1], registry=registry)
+        if to_unit is None:
+            cmks_in_unit = current_mks in unit.dimensions.atoms()
+            cmks_in_unit_system = unit_system.units_map[current_mks]
+            cmks_in_unit_system = cmks_in_unit_system is not None
+            if cmks_in_unit and cmks_in_unit_system:
+                em_map = (unit_system[unit.dimensions], unit, 1.0)
+            else:
+                em_map = (None, em_unit, em_info[2])
+        elif to_unit.dimensions == em_unit.dimensions:
+            em_map = (to_unit, em_unit, em_info[2])
     if em_map:
         return em_map
-    for unit in unit_expr.atoms():
-        if unit.is_Number:
+    for unit_atom in unit.expr.atoms():
+        if unit_atom.is_Number:
             pass
-        base_unit = _get_em_base_unit(str(unit))
-        if base_unit in em_conversions:
-            raise MKSCGSConversionError(unit_expr)
+        bu = str(unit_atom)
+        budims = Unit(bu, registry=registry).dimensions
+        if budims in em_conversions:
+            conv_unit = em_conversions[budims][1]
+            if to_unit is not None:
+                for to_unit_atom in to_unit.expr.atoms():
+                    bou = str(to_unit_atom)
+                    if bou == conv_unit:
+                        raise MKSCGSConversionError(unit)
+            else:
+                raise MKSCGSConversionError(unit)
     return em_map
 
 
@@ -888,82 +895,6 @@ def _get_unit_data_from_expr(unit_expr, unit_symbol_lut):
                          "objects." % str(unit_expr))
 
 
-def _lookup_unit_symbol(symbol_str, unit_symbol_lut):
-    """
-    Searches for the unit data tuple corresponding to the given symbol.
-
-    Parameters
-    ----------
-    symbol_str : str
-        The unit symbol to look up.
-    unit_symbol_lut : dict
-        Dictionary with symbols as keys and unit data tuples as values.
-
-    """
-    if symbol_str in unit_symbol_lut:
-        # lookup successful, return the tuple directly
-        return unit_symbol_lut[symbol_str]
-
-    # could still be a known symbol with a prefix
-    possible_prefix = symbol_str[0]
-
-    if symbol_str[:2] == 'da':
-        possible_prefix = 'da'
-
-    if possible_prefix in unit_prefixes:
-        # the first character could be a prefix, check the rest of the symbol
-        symbol_wo_pref = symbol_str[1:]
-
-        # deca is the only prefix with length 2
-        if symbol_str[:2] == 'da':
-            symbol_wo_pref = symbol_str[2:]
-            possible_prefix = 'da'
-
-        unit_is_si_prefixable = (symbol_wo_pref in unit_symbol_lut and
-                                 symbol_wo_pref in prefixable_units)
-
-        if unit_is_si_prefixable is True:
-            # lookup successful, it's a symbol with a prefix
-            unit_data = unit_symbol_lut[symbol_wo_pref]
-            prefix_value = unit_prefixes[possible_prefix]
-
-            if possible_prefix in latex_prefixes:
-                latex_repr = symbol_str.replace(
-                    possible_prefix, '{'+latex_prefixes[possible_prefix]+'}')
-            else:
-                # Need to add some special handling for comoving units
-                # this is fine for now, but it wouldn't work for a general
-                # unit that has an arbitrary LaTeX representation
-                if symbol_wo_pref != 'cm' and symbol_wo_pref.endswith('cm'):
-                    sub_symbol_wo_prefix = symbol_wo_pref[:-2]
-                    sub_symbol_str = symbol_str[:-2]
-                else:
-                    sub_symbol_wo_prefix = symbol_wo_pref
-                    sub_symbol_str = symbol_str
-                latex_repr = unit_data[3].replace(
-                    '{' + sub_symbol_wo_prefix + '}',
-                    '{' + sub_symbol_str + '}')
-
-            # Leave offset and dimensions the same, but adjust scale factor and
-            # LaTeX representation
-            ret = (unit_data[0] * prefix_value, unit_data[1], unit_data[2],
-                   latex_repr)
-
-            unit_symbol_lut[symbol_str] = ret
-
-            return ret
-
-    # no dice
-    if symbol_str.startswith('code_'):
-        raise UnitParseError(
-            "Code units have not been defined. \n"
-            "Try creating the array or quantity using ds.arr or ds.quan "
-            "instead.")
-    else:
-        raise UnitParseError("Could not find unit symbol '%s' in the provided "
-                             "symbols." % symbol_str)
-
-
 def _validate_dimensions(dimensions):
     if isinstance(dimensions, Mul):
         for dim in dimensions.args:
@@ -1057,10 +988,8 @@ def define_unit(symbol, value, tex_repr=None, offset=None, prefixable=False,
                                "(value, unit) tuple!")
     base_value = float(value.in_base(unit_system='mks'))
     dimensions = value.units.dimensions
-    registry.add(symbol, base_value, dimensions, tex_repr=tex_repr,
-                 offset=offset)
-    if prefixable:
-        prefixable_units.append(symbol)
+    registry.add(symbol, base_value, dimensions, prefixable=prefixable,
+                 tex_repr=tex_repr, offset=offset)
     if registry is default_unit_registry:
         u = Unit(symbol, registry=registry)
         setattr(unyt, symbol, u)
